/*
 * Project name:
    SMS Control
 * Copyright:
     (c) MikroElektronika, 2011.
 * Revision History:
     20110516:
       - initial release(SZ);
 * Description:
      Controlling 4 Relays over SMS and retreiving status SMS containing state of relays, 4 digital inputs,
      2 analog inputs, Temperature and Humidity readings.
 * Test configuration:
     MCU:             PIC18F87J60
                      http://ww1.microchip.com/downloads/en/DeviceDoc/39762f.pdf
     Dev.Board:       PICPLC4 v6
                      ac:PICPLC4v6
                      http://www.mikroe.com/eng/products/view/420/picplc4-v6-plc-system/
     Oscillator:      HS, 25.0000 MHz
     Ext. Modules:    SHT1X Board at PORTC
                      ac:SHT1X_Board
                      http://www.mikroe.com/eng/products/view/367/sht1x-board/
                      Potentiometer Board at PORTA
                      ac:Potentiometer_Board
                      http://www.mikroe.com/eng/products/view/195/potentiometer-board/
                      EasyInput Board at PORTH
                      ac:EasyInput_Board
                      http://www.mikroe.com/eng/products/view/140/easyinput-board/
                      EasyTest Board at PORTH
                      ac:EasyTest_Board
                      http://www.mikroe.com/eng/products/view/183/easytest-board/
     SW:              mikroC PRO for PIC
                      http://www.mikroe.com/eng/products/view/7/mikroc-pro-for-pic/
 * NOTES:
     - THIS PROGRAM IS BROUGHT TO YOU FREE OF CHARGE AND "AS IS" (WITHOUT WARRANTY OF ANY KIND).
       THE ENTIRE RISK AS TO THE QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH YOU. SHOULD THE
       PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING, REPAIR OR CORRECTION.
     - PICPLC4 v6: Turn on GSM switches SW6.1, SW6.3, SW6.5, SW6.6,  SW6.7 and SW6.8.
       (pull-up the SDA pin and pull down the Digital input pins are necessary).
     - Due different mobile operators some adjustments may be required, please consult Telit AT commands
       reference guide at http://www.telit.com/module/infopool/download.php?id=542
     - The Relay control SMS messages (which GM862 receives) should contain commands: "R1ON", "R1OFF", "R2ON" ... and "Status?" (it is case sensitive and without quotes).
       In example, "R1ON"  command turns ON the Relay1 which is on LATE4 pin , "R4OFF" turns OFF the Relay4 which is on LATE7 pin  ... etc...
       You can send only the "Status?" inquiry via your SMS (or add it to the SMS containg relay commands) and then you'll receive an INFO about the
       state of relays, 4 digital inputs, 2 analog inputs, Temperature and Humidity readings. An example of one control message is below:
       "R1ON R2OFF R3ON R4OFF Status?"  (Note that there is no need for separaion chars (spaces in this case)).
       Valid message is also: "R1ONR2OFFR3ONR4OFFStatus?"; just be careful about the case of letters.
       
 */

#include "SHT1X_driver.h"

// Set of AT commands
const char atc0[] = "AT";                        // Every AT command starts with "AT"
const char atc1[] = "ATE0";                      // Disable command echo
const char atc2[] = "AT+CMGF=1";                 // TXT messages
const char atc3[] = "AT+CMGS=\"0123456789\"";    // Send message to cell number : 0123456789 (Enter your cell phone number instead of 0123456789)
const char atc4[] = "AT+CMGR=1";                 // Command for reading message from location 1 from inbox
const char atc5[] = "AT+CMGD=1,4";               // Erasing all messages from inbox
//

// Responses to parse
const GSM_OK = 0;
const GSM_Ready_To_Receive_Message = 1;
const GSM_ERROR = 2;
//

// Relay connections
sbit Relay1 at LATE4_bit;
sbit Relay2 at LATE5_bit;
sbit Relay3 at LATE6_bit;
sbit Relay4 at LATE7_bit;

sbit Relay1_Direction at TRISE4_bit;
sbit Relay2_Direction at TRISE5_bit;
sbit Relay3_Direction at TRISE6_bit;
sbit Relay4_Direction at TRISE7_bit;
//

// Digital inputs
sbit Digital_IN1 at RH0_bit;
sbit Digital_IN2 at RH1_bit;
sbit Digital_IN3 at RH2_bit;
sbit Digital_IN4 at RH3_bit;

sbit Digital_IN1_Direction at TRISH0_bit;
sbit Digital_IN2_Direction at TRISH1_bit;
sbit Digital_IN3_Direction at TRISH2_bit;
sbit Digital_IN4_Direction at TRISH3_bit;
//


// Telit GM862
sbit CTS at RE1_bit;
sbit CTS_Direction at TRISE1_bit;

sbit RTS at RE0_bit;
sbit RTS_Direction at TRISE0_bit;

sbit GM862_ON_OFF at RE2_bit;
sbit GM862_ON_OFF_Direction at TRISE2_bit;
//

char SMS_Message[300]; // SMS Message string

// ADC data strings
char ADC1_data[6];
char ADC2_data[6];
//

// SHT11 connections
sbit SDA_ at RC3_bit;       // Serial data pin
sbit SCL_ at RC2_bit;       // Serial clock pin

sbit SDA_Direction at TRISC3_bit;    // Serial data direction pin
sbit SCL_Direction at TRISC2_bit;    // Serial clock direction pin
//

// SHT11 Temperature & Humidity strings
char Temperature[] = "Ta = 000.00      ";
char Humidity[]    = "Rh = 000.00    ";
//

// State machine control variables
char gsm_state = 0;
char response_rcvd = 0;
short responseID = -1, response = -1, rsp;
//

char status_req = 0; // Status request variable


// Send command or data to the Telit GM862 Module - (const)
void GM862_Send(const char *s)
{
// Send command or data string
   while(*s) {
    UART_Wr_Ptr(*s++);
   }
// Terminatation by CR
   UART_Wr_Ptr(0x0D);
}


// Send command or data to the Telit GM862 Module - (RAM)
void GM862_Send_Ram(char *s1)   //
{
// Send command or data string
   while(*s1) {
    UART_Wr_Ptr(*s1++);
   }
// Terminatation by CR
   UART_Wr_Ptr(0x0D);
}


// Get GSM response, if there is any
short Get_response() {
    if (response_rcvd) {
      response_rcvd = 0;
      return responseID;
    }
    else
      return -1;
}


// Wait for GSM response
void Wait_response(char rspns) {
  while (Get_response() != rspns)
    ;
}


// Compose Status SMS
unsigned ComposeMessage(char* Message);

// Send Status SMS
void Send_Msg(char* Msg){
//       GM862_Send(atc2);
//       Wait_response(GSM_OK);
       GM862_Send(atc3);
       Wait_response(GSM_Ready_To_Receive_Message);

       GM862_Send_Ram(Msg);
       UART_Wr_Ptr(0x1A);
       UART_Wr_Ptr(0x0D);
       Wait_response(GSM_OK);
}

// Send status SMS to the cell phone number defined by the atc3 const string
void Send_Status(){
 ComposeMessage(SMS_Message);
 Send_Msg(SMS_Message);
}

// 5sec pause
void Wait(){
   Delay_ms(5000);
}


// Main
void main(){
  SWDTEN_bit = 0;       // Disable WDT
  ADCON1 = 0x0D;        // Configure AN pins as digital (all but AN0 and AN1)
  CMCON  |= 7;          // Disable comparators
  
  TRISA0_bit = 1;
  TRISA1_bit = 1;
  
// Initialy, relays are turned off
  Relay1 = 0;
  Relay2 = 0;
  Relay3 = 0;
  Relay4 = 0;
//

  Relay1_Direction = 0;
  Relay2_Direction = 0;
  Relay3_Direction = 0;
  Relay4_Direction = 0;

// Digital inputs must be pulled down
  Digital_IN1_Direction = 1;
  Digital_IN2_Direction = 1;
  Digital_IN3_Direction = 1;
  Digital_IN4_Direction = 1;
//

// Setup interrupts
  RC1IE_bit = 1;        // Enable Rx1 intterupts
  PEIE_bit = 1;
  GIE_bit  = 1;
//

  RTS_Direction  = 0;   // RTS pin
  GM862_ON_OFF = 0;
  GM862_ON_OFF_Direction = 0;

  Delay_ms(500);

 // Set RTS pin to zero (we will use only RX i TX)
  RTS = 0;

// Turn on the GM862 module
  GM862_ON_OFF = 1;
  Delay_ms(2500);
  GM862_ON_OFF = 0;
//

  UART1_Init(19200);
  ADC_Init();
  Delay_ms(200);

  Wait(); Wait(); Wait();    // Wait a while till the GSM network is configured

// Negotiate baud rate
  while(1) {
    GM862_Send(atc0);                            // Send "AT" string until GSM862 sets up its baud rade
    Delay_ms(100);                               // and gets it correctly
    if (Get_response() == GSM_OK)                // If GSM862 says "OK" on our baud rate we program can continue
      break;
  }
//

 GM862_Send(atc1);         // Disable command echo
 Wait_response(GSM_OK);

 GM862_Send(atc2);         // Set message type as TXT
 Wait_response(GSM_OK);

 GM862_Send(atc5);         // Delete all messages (if any)
 Wait_response(GSM_OK);

 Send_Status();            // Send Status after power-up
 Wait();

  while(1) {
    GM862_Send(atc4);      // Read SMS message on location 1

    while (1){
       rsp = Get_response();
       if (rsp == GSM_OK || rsp == GSM_ERROR) break;
    }

    if (status_req){       // Send status SMS if it's been requested
      status_req = 0;
      Send_Status();
     }

    if (rsp == GSM_OK){
    Wait();
    GM862_Send(atc5);      // Delete all messages (if any)

    while (1){
      rsp = Get_response();
      if (rsp == GSM_OK) break;
     }

    }
     Wait();
 }
}



/******************************************************************************/

// Compose Status SMS
unsigned ComposeMessage(char* Message){
  unsigned adc_value1, adc_value2;

  RC1IE_bit = 0;                // Disable Rx1 intterupts

  Message[0] = '\0';

  // SMS header
  strcat(Message, "INFO:");
  strcat(Message, "\r\n");      // Add new line (CR + LF)
  //

  // Add the status of Relays to the SMS Message
  if (Relay1)
   strcat(Message, "R1 - ON");
  else
   strcat(Message, "R1 - OFF");
  strcat(Message, "\r\n");      // Add new line (CR + LF)

  if (Relay2)
   strcat(Message, "R2 - ON");
  else
   strcat(Message, "R2 - OFF");
  strcat(Message, "\r\n");      // Add new line (CR + LF)

  if (Relay3)
   strcat(Message, "R3 - ON");
  else
   strcat(Message, "R3 - OFF");
  strcat(Message, "\r\n");      // Add new line (CR + LF)

  if (Relay4)
   strcat(Message, "R4 - ON");
  else
   strcat(Message, "R4 - OFF");
  strcat(Message, "\r\n");      // Add new line (CR + LF)
  //

  // Add the status of Digital Inputs to the SMS Message
  if (Button(&PORTH, 0, 1, 1))  // Digital_IN1
   strcat(Message, "DI1 - Hi");
  else
   strcat(Message, "DI1 - Lo");
  strcat(Message, "\r\n");      // Add new line (CR + LF)

  if (Button(&PORTH, 1, 1, 1))  // Digital_IN2
   strcat(Message, "DI2 - Hi");
  else
   strcat(Message, "DI2 - Lo");
  strcat(Message, "\r\n");      // Add new line (CR + LF)

  if (Button(&PORTH, 2, 1, 1))  // Digital_IN3
   strcat(Message, "DI3 - Hi");
  else
   strcat(Message, "DI3 - Lo");
  strcat(Message, "\r\n");      // Add new line (CR + LF)

  if (Button(&PORTH, 3, 1, 1))  // Digital_IN4
   strcat(Message, "DI4 - Hi");
  else
   strcat(Message, "DI4 - Lo");
  strcat(Message, "\r\n");      // Add new line (CR + LF)
  //

  // Adding ADC values to the SMS
  adc_value1 = ADC_Get_Sample(0);  // RA0
  adc_value2 = ADC_Get_Sample(1);  // RA1
  WordToStr(adc_value1, ADC1_data);
  WordToStr(adc_value2, ADC2_data);
  strcat(Message, "AN1 - ");  // AN1
  strcat(Message, ADC1_data); // Add ADC1 data
  strcat(Message, "\r\n");    // Add new line (CR + LF)
  strcat(Message, "AN2 - ");  // AN1
  strcat(Message, ADC2_data); // Add ADC1 data
  strcat(Message, "\r\n");    // Add new line (CR + LF)
  //

  // Adding temp and humidity informations (SHT1X)
  Read_SHT1X();
  strcat(Message, Temperature); // Add Temperature data
  strcat(Message, "\r\n");      // Add new line (CR + LF)
  strcat(Message, Humidity);    // Add Humidity data
  strcat(Message, "\r\n");      // Add new line (CR + LF)
  //

  // SMS footer
  strcat(Message, "End.");
  strcat(Message, "\r\n");      // Add new line (CR + LF)
  //

  RC1IE_bit = 1;                // Enable Rx1 intterupts

  return strlen(Message);
}



// Reading the data from UART in the interuppt routine
void interrupt(){
char tmp;
  if (RCIF_bit == 1) {                           // Do we have uart rx interrupt request?
    tmp = UART_Rd_Ptr();                         // Get received byte

// Process reception through state machine
// We are parsing only "OK" and "> " responses
    switch (gsm_state) {
      case  0: {
                response = -1;                   // Clear response
                if (tmp == 'O')                  // We have 'O', it could be "OK"
                  gsm_state = 1;                 // Expecting 'K'
                if (tmp == '>')                  // We have '>', it could be "> "
                  gsm_state = 10;                // Expecting ' '
                if (tmp == 'E')                  // We have 'E', it could be "> "
                  gsm_state = 30;                // Expecting 'R'
                if (tmp == 'S')                  // We have 'S', it could be "Status?"
                  gsm_state = 40;                // Expecting 't'
                if (tmp == 'R')                  // We have 'R', it could be "RxON" or "RxOFF"
                  gsm_state = 50;                // Expecting x = (1,...,4)
                break;
               }
      case  1: {
                if (tmp == 'K') {                // We have 'K' ->
                  response = GSM_OK;             // We have "OK" response
                  gsm_state = 20;                // Expecting CR+LF
                }
                else
                  gsm_state = 0;                 // Reset state machine
                break;
               }
      case 10: {
                if (tmp == ' ') {
                  response_rcvd = 1;             // We have "> " response
                  response = GSM_Ready_To_Receive_Message; // Set reception flag
                  responseID = response;         // Set response ID
                }
                gsm_state = 0;                   // Reset state machine
                break;
                }

      case 20: {
                if (tmp == 13)                   // We have 13, it could be CR+LF
                  gsm_state = 21;                // Expecting LF
                else
                  gsm_state = 0;                 // Reset state machine
                break;
               }
      case 21: {
                if (tmp == 10) {                 // We have LF, response is complete
                  response_rcvd = 1;             // Set reception flag
                  responseID = response;         // Set response ID
                }
                gsm_state = 0;                   // Reset state machine
                break;
               }

      case 30: {
                if (tmp == 'R')                  // We have 'R', it could be "ERROR"
                  gsm_state = 31;                // Expecting 'R'
                else
                  gsm_state = 0;                 // Reset state machine
                break;
               }
      case 31: {
                if (tmp == 'R')                  // We have 'R', it could be "ERROR"
                  gsm_state = 32;                // Expecting 'O'
                else
                  gsm_state = 0;                 // Reset state machine
                break;
               }
      case 32: {
                if (tmp == 'O')                  // We have 'O', it could be "ERROR"
                  gsm_state = 33;                // Expecting 'R'
                else
                  gsm_state = 0;                 // Reset state machine
                break;
               }
      case 33: {
                if (tmp == 'R'){                 // We have 'R'
                  response_rcvd = 1;             // We have "ERROR" response
                  response = GSM_ERROR;          // Set reception flag
                  responseID = response;         // Set response ID
                }
                gsm_state = 0;                   // Reset state machine
                break;
                }

      case 40: {
                if (tmp == 't')                  // We have 't', it could be "Status?"
                  gsm_state = 41;                // Expecting 'a'
                else
                  gsm_state = 0;                 // Reset state machine
                break;
               }
      case 41: {
                if (tmp == 'a')                  // We have 'a', it could be "Status?"
                  gsm_state = 42;                // Expecting 't'
                else
                  gsm_state = 0;                 // Reset state machine
                break;
               }
      case 42: {
                if (tmp == 't')                  // We have 't', it could be "Status?"
                  gsm_state = 43;                // Expecting 'u'
                else
                  gsm_state = 0;                 // Reset state machine
                break;
               }

       case 43: {
                if (tmp == 'u')                  // We have 'u', it could be "Status?"
                  gsm_state = 44;                // Expecting 's'
                else
                  gsm_state = 0;                 // Reset state machine
                break;
               }

       case 44: {
                if (tmp == 's')                  // We have 's', it could be "Status?"
                  gsm_state = 45;                // Expecting '?'
                else
                  gsm_state = 0;                 // Reset state machine
                break;
               }

       case 45: {
                if (tmp == '?'){                 // We have '?'
                status_req = 1;                  // Status has been requested!
                }
                gsm_state = 0;                   // Reset state machine
                break;
                }

       case 50: {
                if (tmp == '1')                  // We have '1', it could be "R1ON" or "R1OFF"
                  gsm_state = 51;                // Expecting 'O'
                else
                if (tmp == '2')                  // We have '2', it could be "R2ON" or "R2OFF"
                  gsm_state = 60;                // Expecting 'O'
                else
                if (tmp == '3')                  // We have '3', it could be "R3ON" or "R3OFF"
                  gsm_state = 70;                // Expecting 'O'
                else
                if (tmp == '4')                  // We have '4', it could be "R4ON" or "R4OFF"
                  gsm_state = 80;                // Expecting 'O'
                else
                  gsm_state = 0;                 // Reset state machine
                break;
               }
       case 51: {
                if (tmp == 'O')                  // We have 'O', it could be "R1ON" or "R1OFF"
                  gsm_state = 52;                // Expecting 'N' or 'F'
                else
                  gsm_state = 0;                 // Reset state machine
                break;
               }
       case 52: {
                if (tmp == 'N'){                 // We have 'N'
                  Relay1 = 1;                    // We've got the R1ON command!
                  gsm_state = 0;}
                else
                if (tmp == 'F')                  // We have 'F', it could be "R1OFF"
                  gsm_state = 53;                // Expecting 'F'
                else
                  gsm_state = 0;                 // Reset state machine
                break;
               }

       case 53: {
                if (tmp == 'F'){                 // We have 'F'
                  Relay1 = 0;                    // We've got the R1OFF command!
                  gsm_state = 0;}
                else
                gsm_state = 0;
                break;
               }

       case 60: {
                if (tmp == 'O')                  // We have 'O', it could be "R2ON" or "R2OFF"
                  gsm_state = 61;                // Expecting 'N' or 'F'
                else
                  gsm_state = 0;                 // Reset state machine
                break;
               }
       case 61: {
                if (tmp == 'N'){                 // We have 'N'
                  Relay2 = 1;                    // We've got the R2ON command!
                  gsm_state = 0;}
                else
                if (tmp == 'F')                  // We have 'F', it could be "R2OFF"
                  gsm_state = 62;                // Expecting 'F'
                else
                  gsm_state = 0;                 // Reset state machine
                break;
               }

       case 62: {
                if (tmp == 'F'){                 // We have 'F'
                  Relay2 = 0;                    // We've got the R2OFF command!
                  gsm_state = 0;}
                else
                gsm_state = 0;
                break;
               }

       case 70: {
                if (tmp == 'O')                  // We have 'O', it could be "R3ON" or "R3OFF"
                  gsm_state = 71;                // Expecting 'N' or 'F'
                else
                  gsm_state = 0;                 // Reset state machine
                break;
               }
       case 71: {
                if (tmp == 'N'){                 // We have 'N'
                  Relay3 = 1;                    // We've got the R3ON command!
                  gsm_state = 0;}
                else
                if (tmp == 'F')                  // We have 'F', it could be "R3OFF"
                  gsm_state = 72;                // Expecting 'F'
                else
                  gsm_state = 0;                 // Reset state machine
                break;
               }

       case 72: {
                if (tmp == 'F'){                 // We have 'F'
                  Relay3 = 0;                    // We've got the R3OFF command!
                  gsm_state = 0;}
                else
                gsm_state = 0;
                break;
               }

       case 80: {
                if (tmp == 'O')                  // We have 'O', it could be "R4ON" or "R4OFF"
                  gsm_state = 81;                // Expecting 'N' or 'F'
                else
                  gsm_state = 0;                 // Reset state machine
                break;
               }
       case 81: {
                if (tmp == 'N'){                 // We have 'N'
                  Relay4 = 1;                    // We've got the R4ON command!
                  gsm_state = 0;}
                else
                if (tmp == 'F')                  // We have 'F', it could be "R3OFF"
                  gsm_state = 82;                // Expecting 'F'
                else
                  gsm_state = 0;                 // Reset state machine
                break;
               }

       case 82: {
                if (tmp == 'F'){                 // We have 'F'
                  Relay4 = 0;                    // We've got the R4OFF command!
                  gsm_state = 0;}
                else
                 gsm_state = 0;
                break;
               }

      default: {                                 // Unwanted character
                gsm_state = 0;                   // Reset state machine
                break;
               }
    }

 }
}

/******************************************************************************/